# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__all__ = [
    "__packagename__",
    "__version__",
    "__summary__",
    "__url__",
    "__author__",
    "__email__",
    "__license__"
]

# The name of the package 
__packagename__ = "holoeye_embedded"


__version__ = "0.1.0"

# A one-line summary of what the package does. It's like the first line of a doc string. 
__summary__ = "An alternative interface to official API of the Holoeye PLUTO-2 Spatial Light Modulator in order to control multiple devices through RNDIS network rather than classic HDMI, though you don't need to buy multiple vide cards for communicating to all your beloved SLMs."

# The URL of the package's home page 
__url__ = "https://github.com/ehsan2754/HOLOEYE-PLUTO-2-EMBEDDED"

# The name of the author 
__author__ = "Ehsan Shaghaei"

# The email address of the author. (PEP 241 mentions that this might be used as a unique key in some catalog of packages, but I don't know if it actually is.) 
__email__ = "e.shaghaei@innopolis.university"

# PEP 241 says to put the name of the license here, but I don't think that's a good idea. The name doesn't identify a specific license. There are several Zope licenses and several PSF licenses. I recommend putting the URL of the license. 
__license__ = "MIT"
